/**
 * Copyright (c) 2007-2012, Kaazing Corporation. All rights reserved.
 */

package com.kaazing.gateway.client;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.dom.client.HasNativeEvent;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.dom.client.NativeEvent;

public class WebSocket {

	/* ReadyState constants */
	public final static class ReadyState {
		public static final int	CONNECTING = 0;
		public static final int	OPEN = 1;
		public static final int	CLOSING = 2;
		public static final int	CLOSED = 3;
	}
	
    private WebSocketJSPeer peer;
    OpenHandler openHandler;
    MessageHandler messageHandler;
    CloseHandler closeHandler;
    ErrorHandler errorHandler;

    /**
     * Creates a new WebSocket instance and connects to the remote socket location.
     *
     * @param {String} location      The socket location
     * @param {String} [protocol]    The subprotocol to be communicated over the WebSocket
     *
     * @constructor
     * @name WebSocket
     *
     * @class  WebSocket provides a text-based socket.
     */
    public WebSocket(String URL, String[] protocols) throws WebSocketException {
        this.peer = WebSocketJSPeer.create(this, URL, protocols);
    }

    public WebSocket(String URL) throws WebSocketException {
        this(URL, null);
    }

    protected WebSocket(WebSocketFactory factory, String URL, String[] protocols) throws WebSocketException {
        this.peer = WebSocketJSPeer.createWebSocket(this, factory, URL, protocols);
    }
    /**
     * The URL with which the WebSocket was constructed.
     */
    public String getURL() {
        return this.peer.getURL();
    }

    /**
     * The ready state indicates the connection status.
     */
    public int getReadyState() {
        return this.peer.getReadyState();
    }

    /**
     * The number of bytes queued to be sent
     */
    public int getBufferedAmount() {
        return this.peer.getBufferedAmount();
    }

    /**
     * message event data type for binary messages
     * valid values are: "bytebuffer", "blob" and "arraybuffer"
     */
    public String binaryType() {
    	return this.peer.binaryType();
    }
    
    public void binaryType(String newBinaryType) {
    	this.peer.binaryType(newBinaryType);
    }
    
    /**
     * Disconnects the remote socket location.
     *
     * @return {void}
     *
     * @public
     * @function
     * @name close
     * @memberOf WebSocket
     */
    public void close() {
        this.peer.close();
    }
    
    /**
     * Disconnects the remote socket location with code and reason.
     *
     ** @param {int} code   close code
     ** @param {String} reason   the close reason
     *
     * @return {void}
     *
     * @public
     * @function
     * @name close
     * @memberOf WebSocket
     */
    public void close(int code, String reason) {
        this.peer.close(code, reason);
    }
    

    /**
     * Sends text-based data to the remote socket location.
     *
     * @param {JavaScriptObject} data   the data payload
     *
     * @return {void}
     *
     * @public
     * @function
     * @name send
     * @memberOf WebSocket
     */
    public void send(String data) {
        this.peer.send(data);
    }
    
    /**
     * Sends binary-based data to the remote socket location.
     *
     * @param {ByteBuffer} data   the data payload
     *
     * @return {void}
     *
     * @public
     * @function
     * @name send
     * @memberOf WebSocket
     */
    public void send(ByteBuffer data) {
        this.peer.send(data);
    }
    

    /**
     * The onopen handler is called when the connection is established.
     */
    public void addOpenHandler(OpenHandler handler) {
        openHandler = handler;
    }

    public static class OpenEvent {
        OpenEvent() {
        }
    }
    public interface OpenHandler extends EventHandler {
        void onOpen(OpenEvent ev);
    }

    protected void handleOpen() {
        if (openHandler != null)
            openHandler.onOpen(new OpenEvent());
    }

    public static class MessageEvent implements HasNativeEvent {
        protected NativeMessageEvent event;
        protected MessageEvent(NativeMessageEvent event) {
            this.event = event;
        }

        public String getDataType() { return event.getDataType(); };
        public JavaScriptObject getData() { return event.getData(); };
        public String getDataAsString() { return event.getDataAsString(); };
        public ByteBuffer getDataAsByteBuffer() { return event.getDataAsByteBuffer(); };
        public String getOrigin() { return event.getOrigin(); }
        public String getLastEventId() { return event.getLastEventId(); }
        public WindowProxy getSource() { return event.getSource(); }
        public MessagePortArray getMessagePortArray() { return event.getPorts(); }

        @Override
		public NativeEvent getNativeEvent() {
			return event;
		}
    }
    
    /**
     * The onmessage handler is called when data arrives.
     */
    public void addMessageHandler(MessageHandler handler) {
        messageHandler = handler;
    }

    public interface MessageHandler extends EventHandler {
        void onMessage(MessageEvent ev);
    }

    protected void handleMessage(NativeMessageEvent event) {
        if (messageHandler != null)
            messageHandler.onMessage(new MessageEvent(event));
    }

    /**
     * The onclose handler is called when the connection is terminated.
     */
    public void addCloseHandler(CloseHandler handler) {
        closeHandler = handler;
    }

    public static class CloseEvent implements HasNativeEvent {
        protected NativeCloseEvent event;
        protected CloseEvent(NativeCloseEvent ev) {
            this.event = ev;
        }

        public Boolean wasClean() { return event.wasClean(); };
        public String getReason() { return event.getReason(); }
        public int getCode() { return event.getCode(); }

        @Override
		public NativeEvent getNativeEvent() {
			return event;
		}
    }

    public interface CloseHandler extends EventHandler {
        void onClose(CloseEvent ev);
    }

    protected void handleClose(NativeCloseEvent event) {
        if (closeHandler != null)
            closeHandler.onClose(new CloseEvent(event));
    }
    
    /**
     * The onclose handler is called when an error occurs.
     */
    public void addErrorHandler(ErrorHandler handler) {
        errorHandler = handler;
    }

    public static class ErrorEvent {
        ErrorEvent() {
        }
    }
    public interface ErrorHandler extends EventHandler {
        void onError(ErrorEvent ev);
    }

    protected void handleError() {
        if (errorHandler != null)
            errorHandler.onError(new ErrorEvent());
    }
}
