/**
 * Copyright (c) 2007-2013, Kaazing Corporation. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Browser;
using Kaazing.HTML5;
using Kaazing.AMQP;
using Kaazing.HTML5.Util;
using Kaazing.Security;
using System.Windows.Controls.Primitives;

namespace Kaazing.AMQP.Demo
{
    /// <summary>
    /// AMQP Demo Page
    /// </summary>
    public partial class Page : UserControl
    {
        private AmqpClient     client = null;
        private AmqpChannel    publishChannel = null;
        private AmqpChannel    consumeChannel = null;
        private AmqpChannel    txnPublishChannel = null;
        private AmqpChannel    txnConsumeChannel = null;
        private string         exchangeName = "demo_exchange";
        private string         txnExchangeName = "demo_txn_exchange";
        private string         queueName = "queue" + new Random().Next();
        private string         txnQueueName = "txnqueue" + new Random().Next(); 
        private string         routingKey = "broadcastkey";
        private bool           terminated = false;

        private delegate void InvokeDelegate();

        // username and password from Login Popup Page
        public PasswordAuthentication Credentials;
        public System.Threading.AutoResetEvent userInputCompleted = new System.Threading.AutoResetEvent(false);

        /// <summary>
        /// Top level AMQP Client Demo Page
        /// </summary>
        public Page()
        {
            InitializeComponent();

            Uri sourceUri = Application.Current.Host.Source;

            String defaultLocation = "";
            if (sourceUri.Scheme == "file")
            {
                defaultLocation = "ws://localhost:8001";
            }
            else if (sourceUri.Scheme == "http")
            {
                defaultLocation = "ws://" + sourceUri.Host + ":" + sourceUri.Port;
            }
            else if (sourceUri.Scheme == "https")
            {
                defaultLocation = "wss://" + sourceUri.Host + ":" + sourceUri.Port;
            }

            url.Text = defaultLocation + "/amqp";

            UpdateConnectionButtons(false);

            if (Application.Current.InstallState == InstallState.NotInstalled)
            {
                btnInstall.IsEnabled = true;
            }

            // Create handler for authentication
            BasicChallengeHandler basicHandler = ChallengeHandlers.Load<BasicChallengeHandler>(typeof(BasicChallengeHandler));
            basicHandler.LoginHandler = new LoginHandlerDemo(this);
            ChallengeHandlers.Default = basicHandler;
        }

        //Popup login page
        public void PopupLoginPage()
        {
            Popup p = new Popup();

            // Set the Child property of Popup to an instance of LoginPage. 
            LoginPage login = new LoginPage(this);
            p.Child = login;

            // Set where the popup will show up on the screen. 
            p.VerticalOffset = 200;
            p.HorizontalOffset = 200;

            // Open the popup. 
            p.IsOpen = true;
        }

        private void HandleLog(String message)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("LOG: " + message);
            });
        }

        /*
         *  Button click handlers
         */

        private void Connect_Click(object sender, RoutedEventArgs e)
        {
            client = new AmqpClient();
            client.OpenEvent += new AmqpEventHandler(ConnectedHandler);
            client.CloseEvent += new AmqpEventHandler(CloseHandler);
            connection_status.Text = "CONNECTING";

            Log("\n");
            Log("CONNECTING: " + url.Text + " " + username.Text);
            UpdateTransactionButtons(false, true);

            string virtualHost = txtVirtualHost.Text;

            client.Connect(url.Text, virtualHost, username.Text, password.Password);

            exchangeName = txtExchange.Text;
            txnExchangeName = txtTxnExchange.Text;
        }

        private void Disconnect_Click(object sender, RoutedEventArgs e)
        {
            Log("DISCONNECT");
            publishChannel.CloseChannel(0, "", 0, 0);
            consumeChannel.CloseChannel(0, "", 0, 0);
            txnPublishChannel.CloseChannel(0, "", 0, 0);
            txnConsumeChannel.CloseChannel(0, "", 0, 0); 
            Disconnect();
        }

        private void Disconnect()
        {
            terminated = true;
            client.Disconnect();
            client = null;
        }

        private void CreateChannel()
        {
            Log("OPEN: Publish Channel");
            publishChannel = client.OpenChannel();
            publishChannel.OpenEvent += PublishChannelOpenHandler;
            publishChannel.DeclareExchangeEvent += DeclareExchangeOkHandler;
            publishChannel.CloseEvent += ChannelCloseHandler;

            Log("OPEN: Consume Channel");
            consumeChannel = client.OpenChannel();
            consumeChannel.OpenEvent += ConsumeChannelOpenHandler;
            consumeChannel.DeclareQueueEvent += DeclareQueueOkHandler;
            consumeChannel.BindQueueEvent += BindQueueOkHandler;
            consumeChannel.ConsumeEvent += BasicConsumeOkHandler;
            consumeChannel.MessageEvent += MessageHandler;
            consumeChannel.FlowEvent += FlowOkHandler;
            consumeChannel.CancelEvent += CancelOkBasicHandler;
            consumeChannel.CloseEvent += ChannelCloseHandler;

            txnPublishChannel = client.OpenChannel();
            txnPublishChannel.OpenEvent += TxnPublishChannelOpenHandler;
            txnPublishChannel.CommitTransactionEvent += CommitOkTxHandler;
            txnPublishChannel.RollbackTransactionEvent += RollbackOkTxHandler;
            txnPublishChannel.SelectTransactionEvent += SelectOkTxHandler;

            txnConsumeChannel = client.OpenChannel();
            txnConsumeChannel.OpenEvent += TxnConsumeChannelOpenHandler;
            txnConsumeChannel.MessageEvent += TxnMessageHandler;
        }

        /* Click Handler */
        private void SelectTx_Click(object sender, RoutedEventArgs e)
        {
            Log("TXN SELECT/START");
            txnPublishChannel.SelectTx();
        }

        /* Click Handler */
        private void PublishTx_Click(object sender, RoutedEventArgs e)
        {
            if (txtTxnMessage.Text == null || txtTxnMessage.Text.Length == 0)
            {
                HtmlPage.Window.Alert("Please enter valid text for AMQP message");
                return;
            }

            TxnPublishBasic(txtTxnMessage.Text);
        }

        /* Click Handler */
        private void CommitTx_Click(object sender, RoutedEventArgs e)
        {
            Log("TXN COMMIT");
            txnPublishChannel.CommitTx();
        }

        /* Click Handler */
        private void RollbackTx_Click(object sender, RoutedEventArgs e)
        {
            Log("TXN ROLLBACK");
            txnPublishChannel.RollbackTx();
        }

        /* Click Handler */
        private void Publish_Click(object sender, RoutedEventArgs e)
        {
            if (txtMessage.Text == null || txtMessage.Text.Length == 0)
            {
                HtmlPage.Window.Alert("Please enter valid text for AMQP message");
                return;
            }

            PublishBasic(txtMessage.Text);
        }

        /* Click Handler */
        private void Flow_On_Click(object sender, RoutedEventArgs e)
        {
            consumeChannel.FlowChannel(true);
        }

        /* Click Handler */
        private void Flow_Off_Click(object sender, RoutedEventArgs e)
        {
            consumeChannel.FlowChannel(false);
        }

        private void ClearLog_Click(object sender, RoutedEventArgs e)
        {
            ClearLog();
        }

        private void Install_Click(object sender, RoutedEventArgs e)
        {
            btnInstall.IsEnabled = false;

            try
            {
                Application.Current.Install();
            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("The application is already installed.");
            }
        }

        /////////////////////////////////////////////////////////////////////////////
        ///All registered Event Handlers.
        /////////////////////////////////////////////////////////////////////////////
        private void CloseHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("DISCONNECTED");
                terminated = true;
                client = null;
                connection_status.Text = "DISCONNECTED";
                UpdateConnectionButtons(false);

                txtExchange.IsReadOnly = false;
                txtTxnExchange.IsReadOnly = false;
             });

        }

        private void ConnectedHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("CONNECTED");
                connection_status.Text = "CONNECTED";

                CreateChannel();

                // Ready to process messages
                UpdateConnectionButtons(true);

                txtExchange.IsReadOnly = true;
                txtTxnExchange.IsReadOnly = true;
            });
        }

        /*
         * Publish Channel Handlers
         */
        private void PublishChannelOpenHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("OPENED: Publish Channel");

                publishChannel.DeclareExchange(exchangeName, "fanout", false, false, false, null);
            });
        }

        private void TxnPublishChannelOpenHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                txnPublishChannel.DeclareExchange(txnExchangeName, "fanout", false, false, false, null);
            });
        }

        private void PublishBasic(string text)
        {
            ByteBuffer buffer = new ByteBuffer();
            buffer.PutString(text, System.Text.Encoding.UTF8);
            buffer.Flip();
            AmqpProperties amqpProperties = new AmqpProperties();
            amqpProperties.MessageId = "abcdxyz1234pqr";
            amqpProperties.CorrelationId = "23456";
            amqpProperties.UserId = username.Text;
            amqpProperties.ContentType = AmqpProperties.TEXT_PLAIN;
            amqpProperties.DeliveryMode = 1;
            amqpProperties.Priority = 6;
            amqpProperties.Timestamp = DateTime.Now.ToLocalTime();
            AmqpArguments customHeaders = new AmqpArguments();
            customHeaders.AddInteger("KZNG_AMQP_KEY1", 100);
            customHeaders.AddLongString("KZNG_AMQP_KEY2", "Custom Header Value");
            amqpProperties.Headers = customHeaders;
            publishChannel.PublishBasic(buffer, amqpProperties, exchangeName, routingKey, false, false);
            Log(amqpProperties.ToString());
            Log("Published Message Properties:");
            Log("MESSAGE PUBLISHED: " + text);
        }

        private void TxnPublishBasic(string text)
        {
            ByteBuffer buffer = new ByteBuffer();
            buffer.PutString(text, System.Text.Encoding.UTF8);
            buffer.Flip();

            AmqpProperties amqpProperties = new AmqpProperties();
            txnPublishChannel.PublishBasic(buffer, amqpProperties, txnExchangeName, routingKey, false, false);
            Log("TXN PUBLISHED MESSAGE: " + text);
        }

        private void DeclareExchangeOkHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("EXCHANGE DECLARED: " + exchangeName);
            });
        }

        /*
         * Consume Channel Handlers
         */
        private void ConsumeChannelOpenHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("OPENED: Consume Channel");

                consumeChannel.DeclareQueue(queueName, false, false, false, false, false, null)
                              .BindQueue(queueName, exchangeName, routingKey, false, null)
                              .ConsumeBasic(queueName, routingKey, false, false, false, false, null);
            });
        }

        private void TxnConsumeChannelOpenHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                txnConsumeChannel.DeclareQueue(txnQueueName, false, false, false, false, false, null)
                                 .BindQueue(txnQueueName, txnExchangeName, routingKey, false, null)
                                 .ConsumeBasic(txnQueueName, routingKey, false, false, false, false, null);
            });
        }


        private void DeclareQueueOkHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("QUEUE DECLARED: " + queueName);
            });
        }

        private void BindQueueOkHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("QUEUE BOUND: " + exchangeName + "  -   " + queueName);
            });
        }

        private void BasicConsumeOkHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("READY TO CONSUME FROM QUEUE: " + queueName);
            });
        }

        private void ErrorHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("ERROR");
            });
        }

        private void ChannelCloseHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("CHANNEL CLOSED");
            });
        }

        private void CancelOkBasicHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("BASIC CANCELLED");
            });
        }

        private void FlowOkHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke((InvokeDelegate)(() =>
            {
                Log("FLOW: " + (e.IsFlowActive ? "ON" : "OFF"));
            }));
        }

        private void CommitOkTxHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("TXN COMMITTED");
                UpdateTransactionButtons(false, true);
            });
        }

        private void RollbackOkTxHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("TXN ROLLEDBACK");
                UpdateTransactionButtons(false, true);
            });
        }

        private void SelectOkTxHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Log("TXN SELECTED/STARTED");
                UpdateTransactionButtons(true, true);
            });
        }

        private void MessageHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                ByteBuffer buf = e.Body;
                string message = buf.GetString(System.Text.Encoding.UTF8);
                Log(e.AmqpProperties.ToString());
                Log("Consumed Message Properties:");
                Log("MESSAGE CONSUMED: " + message);

                // Explicitly acknowledge the message as we are passing
                // 'false' as the value of the 'noAck' parameter in
                // the consumeChannel.ConsumeBasic() call.
                long dt = Convert.ToInt64(e.Arguments["deliveryTag"]);
                ((AmqpChannel)sender).AckBasic(dt, true);

            });
        }

        private void TxnMessageHandler(object sender, AmqpEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                ByteBuffer buf = e.Body;
                string message = buf.GetString(System.Text.Encoding.UTF8);
                Log("TXN MESSAGE CONSUMED: " + message);

                // Explicitly acknowledge the message as we are passing
                // 'false' as the value of the 'noAck' parameter in
                // the txnConsumeChannel.ConsumeBasic() call.
                long dt = Convert.ToInt64(e.Arguments["deliveryTag"]);
                ((AmqpChannel)sender).AckBasic(dt, true);
            });
        }

        private void Log(string message)
        {
            logs.Text = message + "\n" + logs.Text;
            Logger.log("AmqpApp: " + message);
        }

        private void ClearLog()
        {
            logs.Text = "";
        }

        private void UpdateConnectionButtons(bool connected)
        {
            btnConnect.IsEnabled = !connected;
            btnDisConnect.IsEnabled = connected;
            send2.IsEnabled = connected;
            flowoff1.IsEnabled = connected;
            flowon1.IsEnabled = connected;

            UpdateTransactionButtons(false, connected);
        }

        private void UpdateTransactionButtons(bool transacted, bool connected)
        {
            if (connected)
            {
                selecttx.IsEnabled = !transacted;
                committx.IsEnabled = transacted;
                send1.IsEnabled = transacted;
                rollbacktx.IsEnabled = transacted;
            }
            else
            {
                selecttx.IsEnabled = false;
                committx.IsEnabled = false;
                send1.IsEnabled = false;
                rollbacktx.IsEnabled = false;
            }
        }
    }
}
